# CommentCustomField
#
# Release 0.01 (Aug 14, 2009)
# Release 0.02 (Aug 17, 2009) add Image Upload.
# Release 0.03 (Sep 10, 2009) fix bug.
# Release 0.04 (Oct 02, 2009) add radio/checkbox/select.
#
# This software is provided as-is. You may use it for commercial or
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2004-2009 Yujiro Araki

package MT::Plugin::CommentCustomField;

use strict;
use warnings;
use CGI;
use MT;
use base qw( MT::Plugin );
use MT::Template;
use File::Basename;
use MT::Util qw( remove_html multi_iter encode_js is_valid_email is_url encode_url decode_url epoch2ts ts2epoch format_ts dirify encode_html );

our $VERSION = '0.04';
our $SCHEMA_VERSION = '0.3';

my $plugin = __PACKAGE__->new ({
    id   => 'commentcustomfield',
    name => 'CommentCustomField',
    description => '<MT_TRANS phrase="CommentCustomField.">',
    doc_link => 'http://www.koikikukan.com/archives/2009/08/14-025555.php',
    author_name => 'Yujiro Araki',
    author_link => 'http://www.koikikukan.com/',
    l10n_class => 'CommentCustomField::L10N',
    blog_config_template => 'blog_config.tmpl',
    settings => new MT::PluginSettings([
        ['use_comment_customfield_text', { Default => 1 }],
        ['use_comment_customfield_textarea', { Default => 0 }],
        ['use_comment_customfield_radio', { Default => 0 }],
        ['use_comment_customfield_checkbox', { Default => 0 }],
        ['use_comment_customfield_select', { Default => 0 }],
        ['comment_image_upload_path', { Default => '' }],
        ['comment_image_max_size', { Default => '300' }],
        ['comment_image_add_timestamp', { Default => '1' }],
        ['comment_radio_initial_data', { Default => '' }],
        ['comment_checkbox_initial_data', { Default => '' }],
        ['comment_select_initial_data', { Default => '' }],
        ['comment_checkbox_separator', { Default => ',' }],
        ['comment_select_separator', { Default => ',' }],
    ]),
    version => $VERSION,
    schema_version => $SCHEMA_VERSION,
});
MT->add_plugin($plugin);

sub instance { $plugin }

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'comment' => {
                'custom_text' => 'string(255)',
                'custom_textarea' => 'text',
                'custom_image' => 'integer',
                'custom_radio' => 'string(255)',
                'custom_checkbox' => 'string(255)',
                'custom_select' => 'string(255)',
            },
        },
        callbacks => {
            'MT::App::CMS::template_param.edit_comment' => \&edit_comment_customfield,
            'MT::Comment::pre_save' => \&save_comment_customfield,
        },
        tags => {
            function => {
                'CommentCustomFieldText' => \&comment_customfield_text,
                'CommentCustomFieldTextarea' => \&comment_customfield_textarea,
                'CommentCustomFieldRadio' => \&comment_customfield_radio,
                'CommentCustomFieldCheckbox' => \&comment_customfield_checkbox,
                'CommentCustomFieldSelect' => \&comment_customfield_select,
                'CommentCustomFieldRadioInputValue' => \&comment_customfield_radio_input_value,
                'CommentCustomFieldCheckboxInputValue' => \&comment_customfield_checkbox_input_value,
                'CommentCustomFieldSelectInputValue' => \&comment_customfield_select_input_value,
                'PreviewCommentCustomFieldText' => \&preview_comment_customfield_text,
                'PreviewCommentCustomFieldTextarea' => \&preview_comment_customfield_textarea,
                'PreviewCommentCustomFieldRadioInputValue' => \&preview_comment_customfield_radio_input_value,
                'PreviewCommentCustomFieldCheckboxInputValue' => \&preview_comment_customfield_checkbox_input_value,
                'PreviewCommentCustomFieldSelectInputValue' => \&preview_comment_customfield_select_input_value,
            },
            block => {
                'CommentCustomFieldAsset' => \&comment_customfield_asset,
            },
        },
    });
}

sub save_comment_customfield {
    my ($eh, $comment) = @_;

    my $app = MT->instance;
    return if (!$app->can('param')); 

    if ($app->{query}->param('comment-custom-text')) {
        $comment->custom_text($app->{query}->param('comment-custom-text'));
    }

    if ($app->{query}->param('comment-custom-textarea')) {
        $comment->custom_textarea($app->{query}->param('comment-custom-textarea'));
    }

    if ($app->{query}->param('comment-custom-checkbox')) {
        my @checkbox_list = $app->{query}->param('comment-custom-checkbox');
        my $checkbox_sep = $plugin->get_config_value('comment_checkbox_separator', 'blog:' . $app->blog->id) || ',';
        $comment->custom_checkbox(join($checkbox_sep, @checkbox_list));
    }

    if ($app->{query}->param('comment-custom-radio')) {
        $comment->custom_radio($app->{query}->param('comment-custom-radio'));
    }

    if ($app->{query}->param('comment-custom-select')) {
        my @select_list = $app->{query}->param('comment-custom-select');
        my $select_sep = $plugin->get_config_value('comment_select_separator', 'blog:' . $app->blog->id) || ',';
        $comment->custom_select(join($select_sep, @select_list));
    }

    if ($app->{query}->param("comment-custom-image")) {
        my $asset = _handle_upload('comment-custom-image', require_type => 'image');
        return 1 if !$asset;
        $comment->custom_image($asset->id);
    }

    1;
}

sub comment_customfield_asset {
    my ($ctx, $args, $cond) = @_;

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');

    require MT::Asset;

    my $comment = $ctx->stash('comment');
    my $id = $comment->custom_image;
    return '' unless $id;

    my $asset = MT::Asset->load($id);
    return '' unless $asset;

    my $blog = $ctx->stash('blog');
    return '' if $asset->blog_id != $blog->id;

    local $ctx->{__stash}{asset} = $asset;
    defined(my $out = $builder->build($ctx, $tokens))
        or return $ctx->error($builder->errstr);
    $out;
}

sub comment_customfield_text {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_text;
    }
    else {
        return '';
    }
}

sub comment_customfield_textarea {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_textarea;
    }
    else {
        return '';
    }
}

sub comment_customfield_radio_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $radio_input_value = $plugin->get_config_value('comment_radio_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($radio_input_value) {
        my @radio_list = split(/,/, $radio_input_value);
        my $radio_data;
        for (@radio_list) {
            my $checked = '';
            if (/\*$/) {
                chop;
                $checked = 'checked="checked" ';
            }
            $radio_data .= "<label class=\"input_radio_label\"><input type=\"radio\" class=\"input_radio\" name=\"comment-custom-radio\" value=\"$_\" ${checked}/><span class=\"input_radio_value\">$_</span></label> " ;
        }
        return $radio_data;
    }
    else {
        return '';
    }
}

sub comment_customfield_radio {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_radio;
    }
    else {
        return '';
    }
}

sub comment_customfield_checkbox_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $checkbox_input_value = $plugin->get_config_value('comment_checkbox_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($checkbox_input_value) {
        my @checkbox_list = split(/,/, $checkbox_input_value);
        my $checkbox_data;
        for (@checkbox_list) {
            my $checked = '';
            if (/\*$/) {
                chop;
                $checked = 'checked="checked" ';
            }
            $checkbox_data .= "<label class=\"input_checkbox_label\"><input type=\"checkbox\" class=\"input_checkbox\" name=\"comment-custom-checkbox\" value=\"$_\" ${checked}/><span class=\"input_checkbox_value\">$_</span></label> " ;
        }
        return $checkbox_data;
    }
    else {
        return '';
    }
}

sub comment_customfield_checkbox {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_checkbox;
    }
    else {
        return '';
    }
}

sub comment_customfield_select_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $select_input_value = $plugin->get_config_value('comment_select_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($select_input_value) {
        my @select_list = split(/,/, $select_input_value);
        my $select_data = "<select class=\"input_select\" name=\"comment-custom-select\">";
        for (@select_list) {
            my $selected = '';
            if (/\*$/) {
                chop;
                $selected = ' selected="selected"';
            }
            $select_data .= "<option value=\"$_\"${selected}>$_</option>";
        }
        $select_data .= "</select>";
        return $select_data;
    }
    else {
        return '';
    }
}

sub comment_customfield_select {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_select;
    }
    else {
        return '';
    }
}

sub comment_customfield_image_url {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    if ($comment && $comment->id) {
        return $comment->custom_image_url;
    }
    else {
        return '';
    }
}

sub preview_comment_customfield_text {
    my ($ctx, $args) = @_;
    my $app = MT->instance;
    return $app->param('comment-custom-text');
}

sub preview_comment_customfield_textarea {
    my ($ctx, $args) = @_;
    my $app = MT->instance;
    return $app->param('comment-custom-textarea');
}

sub preview_comment_customfield_radio_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $radio_input_value = $plugin->get_config_value('comment_radio_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($radio_input_value) {
        my @radio_list = split(/,/, $radio_input_value);
        my $radio_data;
        my $input_data = $app->param('comment-custom-radio');
        for (@radio_list) {
            chop if /\*$/;
            my $checked = '';
            if($_ eq $input_data) {
                $checked = 'checked="checked" ';
            }
            $radio_data .= "<label class=\"input_radio_label\"><input type=\"radio\" class=\"input_radio\" name=\"comment-custom-radio\" value=\"$_\" ${checked}/><span class=\"input_radio_value\">$_</span></label> " ;
        }
        return $radio_data;
    }
    else {
        return '';
    }
}

sub preview_comment_customfield_checkbox_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $checkbox_input_value = $plugin->get_config_value('comment_checkbox_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($checkbox_input_value) {
        my @checkbox_list = split(/,/, $checkbox_input_value);
        my $checkbox_data;
        my @input_data = $app->param('comment-custom-checkbox');
        for (@checkbox_list) {
            chop if /\*$/;
            my $checked = '';
            for my $cb (@input_data) {
                if($_ eq $cb) {
                    $checked = 'checked="checked" ';
                }
            }
            $checkbox_data .= "<label class=\"input_checkbox_label\"><input type=\"checkbox\" class=\"input_checkbox\" name=\"comment-custom-checkbox\" value=\"$_\" ${checked}/><span class=\"input_checkbox_value\">$_</span></label> " ;
        }
        return $checkbox_data;
    }
    else {
        return '';
    }
}

sub preview_comment_customfield_select_input_value {
    my ($ctx, $args) = @_;
    my $comment = $ctx->stash('comment');
    my $app = MT->instance;
    my $select_input_value = $plugin->get_config_value('comment_select_initial_data', 'blog:' . $app->blog->id) || ',';
    if ($select_input_value) {
        my @select_list = split(/,/, $select_input_value);
        my $select_data = "<select class=\"input_select\" name=\"comment-custom-select\">";
        my @input_data = $app->param('comment-custom-select');
        for (@select_list) {
            chop if /\*$/;
            my $selected = '';
            for my $sl (@input_data) {
                if($_ eq $sl) {
                    $selected = ' selected="selected"';
                }
            }
            $select_data .= "<option value=\"$_\"${selected}>$_</option>";
        }
        $select_data .= "</select>";
        return $select_data;
    }
    else {
        return '';
    }
}
sub preview_comment_customfield_select {
    my ($ctx, $args) = @_;
    my $app = MT->instance;
    return $app->param('comment-custom-select');
}

sub edit_comment_customfield {
    my ($eh, $app, $param, $tmpl) = @_;
    my ($node, $host_node, $innerHTML);

    my $use_textarea = $plugin->get_config_value('use_comment_customfield_textarea', 'blog:' . $app->blog->id);
    if ($use_textarea) {
      $node = $tmpl->createElement('app:setting');
      $node->setAttribute('id', 'comment_custom_textarea');
      $node->setAttribute('label', $plugin->translate('Comment CustomField Textarea'));
      my $comment_custom_textarea = $param->{custom_textarea};
      $innerHTML = <<HERE;
        <textarea name="comment-custom-textarea" id="comment-custom-textarea" style="width:90%">$comment_custom_textarea</textarea>
HERE
      $node->innerHTML($innerHTML);
      $host_node = $tmpl->getElementById('text');
      $tmpl->insertAfter($node, $host_node);
    }

    my $use_text = $plugin->get_config_value('use_comment_customfield_text', 'blog:' . $app->blog->id);
    if ($use_text) {
      $node = $tmpl->createElement('app:setting');
      $node->setAttribute('id', 'comment_custom_text');
      $node->setAttribute('label', $plugin->translate('Comment CustomField Textfield'));
      my $comment_custom_text = $param->{custom_text};
      $innerHTML = <<HERE;
        <input name="comment-custom-text" id="comment-custom-text" value="${comment_custom_text}" style="width:90%" />
HERE
      $node->innerHTML($innerHTML);
      $host_node = $tmpl->getElementById('text');
      $tmpl->insertAfter($node, $host_node);
    }

    my $use_radio = $plugin->get_config_value('use_comment_customfield_radio', 'blog:' . $app->blog->id);
    if ($use_radio) {
      $node = $tmpl->createElement('app:setting');
      $node->setAttribute('id', 'comment_custom_radio');
      $node->setAttribute('label', $plugin->translate('Comment CustomField Radio'));
      my $comment_custom_radio = $param->{custom_radio};
      $innerHTML = <<HERE;
        <input name="comment-custom-radio" id="comment-custom-radio" value="${comment_custom_radio}" style="width:90%" />
HERE
      $node->innerHTML($innerHTML);
      $host_node = $tmpl->getElementById('text');
      $tmpl->insertAfter($node, $host_node);
    }

    my $use_checkbox = $plugin->get_config_value('use_comment_customfield_checkbox', 'blog:' . $app->blog->id);
    if ($use_checkbox) {
      $node = $tmpl->createElement('app:setting');
      $node->setAttribute('id', 'comment_custom_checkbox');
      $node->setAttribute('label', $plugin->translate('Comment CustomField Checkbox'));
      my $comment_custom_checkbox = $param->{custom_checkbox};
      $innerHTML = <<HERE;
        <input name="comment-custom-checkbox" id="comment-custom-checkbox" value="${comment_custom_checkbox}" style="width:90%" />
HERE
      $node->innerHTML($innerHTML);
      $host_node = $tmpl->getElementById('text');
      $tmpl->insertAfter($node, $host_node);
    }

    my $use_select = $plugin->get_config_value('use_comment_customfield_select', 'blog:' . $app->blog->id);
    if ($use_select) {
      $node = $tmpl->createElement('app:setting');
      $node->setAttribute('id', 'comment_custom_select');
      $node->setAttribute('label', $plugin->translate('Comment CustomField Select'));
      my $comment_custom_select = $param->{custom_select};
      $innerHTML = <<HERE;
        <input name="comment-custom-select" id="comment-custom-select" value="${comment_custom_select}" style="width:90%" />
HERE
      $node->innerHTML($innerHTML);
      $host_node = $tmpl->getElementById('text');
      $tmpl->insertAfter($node, $host_node);
    }
}

# Thanks Community.pm _handle_upload
sub _handle_upload {
    my ($param, %options) = @_;
    my $app = MT->instance;

    my ($fh, $no_upload);
    my $q = CGI->new();
    eval { $fh = $q->upload($param) };
    if ( $@ && $@ =~ /^Undefined subroutine/ ) {
        $fh = $q->param($param);
    }
    $no_upload = !$fh;
    return 0 if $no_upload;

    my $info = $q->uploadInfo($fh);
    my $mimetype;
    if ($info) {
        $mimetype = $info->{'Content-Type'};
    }

    # TCY
    my $maxsize = $plugin->get_config_value('comment_image_max_size', 'blog:' . $app->blog->id) || 300;
    my $size = (stat($fh))[7];
    return 0 if ($size > $maxsize * 1024);

    my $basename = basename($fh);

    $basename =~ s!\\!/!g;    ## Change backslashes to forward slashes
    $basename =~ s!^.*/!!;    ## Get rid of full directory paths
    if ( $basename =~ m!\.\.|\0|\|! ) {
        return $app->errtrans( "Invalid filename '[_1]'", $basename );
    }
    my ($base, $uploaded_path, $ext) = File::Basename::fileparse($basename, '\.[^\.]*');
    # dirify $base
    $base = dirify( $base );

    if ($plugin->get_config_value('comment_image_add_timestamp', 'blog:' . $app->blog->id)) {
        my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = gmtime(time);
        my $timestamp = sprintf("%04d%02d%02d%02d%02d%02d", $year + 1900, $mon +1, $mday, $hour, $min, $sec);
        $base .= '_comment' . $timestamp;
    }

    if (my $asset_type = $options{require_type}) {
        require MT::Asset;
        my $asset_pkg = MT::Asset->handler_for_file($basename);
        if ($asset_type eq 'image') {
            return $app->errtrans( "Please select an image to upload." )
              if !$asset_pkg->isa('MT::Asset::Image');
        }
    }

    my ($blog_id, $blog, $save_path, $local_path, $fmgr, %complements);

    $blog_id    = $app->blog->id;
    $save_path  = '%r'. '/' . $plugin->get_config_value('comment_image_upload_path', 'blog:' . $app->blog->id);
    $save_path .= '/' unless $save_path =~ m|/$|;
    if ($plugin->get_config_value('comment_image_upload_path', 'blog:' . $app->blog->id)) {
        $local_path = File::Spec->catdir(
                 $app->blog->site_path . '/' . $plugin->get_config_value('comment_image_upload_path', 'blog:' . $app->blog->id));
        require MT::FileMgr;
        my $fmgr = MT::FileMgr->new('Local');
        $fmgr->mkpath($local_path);
    } else {
        $local_path = File::Spec->catdir($app->blog->site_path);
    }

    require MT::FileMgr;
    $fmgr = MT::FileMgr->new('Local');
    %complements = (
        Max => ($app->config('UserpicMaxUpload') || 0),
        Square => ($options{square} || 0),
    );

    # Find unique name for the file.
    my $i = 1;
    my $base_copy = $base;
    while ($fmgr->exists(File::Spec->catfile($local_path, $base . $ext))) {
        $base = $base_copy . '_' . $i++;
    }

    my $local_relative = File::Spec->catfile($save_path, $base . $ext);
    my $local = File::Spec->catfile($local_path, $base . $ext);

    require MT::Image;
    require Carp;
    my ($w, $h, $id, $write_file) = MT::Image->check_upload(
        Fh => $fh, Fmgr => $fmgr, Local => $local,
        %complements
    );

    return $app->error(MT::Image->errstr)
        unless $write_file;
    my $umask = oct $app->config('UploadUmask');
    my $old   = umask($umask);
    defined( my $bytes = $write_file->() )
      or return $app->error(
        $app->translate(
            "Error writing upload to '[_1]': [_2]", $local,
            $fmgr->errstr
        )
      );
    umask($old);

    ## Close up the filehandle.
    close $fh;

    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file($local);
    my $is_image  = defined($w)
      && defined($h)
      && $asset_pkg->isa('MT::Asset::Image');
    my $asset;
    $asset = $asset_pkg->new();
    $asset->file_path($local_relative);
    $asset->file_name($base.$ext);
    my $ext_copy = $ext;
    $ext_copy =~ s/\.//;
    $asset->file_ext($ext_copy);
    $asset->blog_id($blog_id);

    my $original = $asset->clone;
    my $url = $local_relative;
    $url  =~ s!\\!/!g;
    $asset->url($url);
    if ($is_image) {
        $asset->image_width($w);
        $asset->image_height($h);
    }
    $asset->mime_type($mimetype);

    $asset->save
        or return $app->error($asset->errstr);
    $asset;
}

1;
